﻿Imports System.Math
Imports System.Windows.Forms.Application
'ロボットＩ／Ｏクラス
'移動ユニットの制御関連
Public Class RoboIO_Move

    '指定座標へ移動の計算のみ行う
    '座標は現在位置が原点。初めに旋回してから直進する。
    'ゴール地点での方向指定はなし。（まっすぐ向かう）
    'taget_x, taget_y   : 移動目標座標（ｍ単位）
    'turn_r             : 旋回半径（ｍ単位）・・・旋回半径の実測値を与える
    'turn_q             : 旋回部分での旋回量（ラジアン）→ＧＰＳで移動方向を求める時に直進開始位置を求めるのに使用
    'end_of_arc()       : 旋回終了地点の相対座標
    Shared Sub go_coo1(ByVal target_x As Double, ByVal target_y As Double, ByVal turn_r As Double, ByRef turn_q As Double, ByRef end_of_arc() As Double, ByRef dist2go As Double)

        Dim dist0, dist1, dist2, center(1, 1), R(1), cross_point(1, 1), coo(1, 1) As Double
        Dim x, y As Double
        Dim calc_2d As Calc_2D

        calc_2d = New Calc_2D

        '前提条件
        '現在のＥＲ０１の位置をＸ，Ｙ＝０，０の原点とする
        '向きは前方をＸ軸の正とする
        'ＥＲ０１の旋回半径（実測値）
        R(0) = turn_r

        '目標が第3、第4象限の場合→右旋回、旋回中心はX軸の正側
        '目標が第1、第2象限の場合→左旋回、旋回中心はX軸の負側
        '旋回中心の座標
        If target_y <= 0 Then
            center(0, 0) = 0.0 'X
            center(0, 1) = -R(0) 'Y
        Else
            center(0, 0) = 0.0 'X
            center(0, 1) = R(0) 'Y
        End If

        '目標から旋回円への接線を求めるため、２円の交点を求める。
        '一つ目の円は、目標と原点を結ぶ直線上で、その距離から旋回円の半径をマイナスした後
        'の数値の半分
        coo(0, 0) = center(0, 0)
        coo(0, 1) = center(0, 1)
        coo(1, 0) = target_x
        coo(1, 1) = target_y

        dist0 = calc_2d.dist2points(coo)
        R(1) = dist0 / 2.0

        '２つ目の円の中心
        center(1, 0) = (target_x - center(0, 0)) / 2 + center(0, 0)
        center(1, 1) = (target_y - center(0, 1)) / 2 + center(0, 1)

        calc_2d.crcir(center, R, cross_point)

        dist1 = (cross_point(0, 0) ^ 2 + cross_point(0, 1) ^ 2) ^ 0.5
        dist2 = (cross_point(1, 0) ^ 2 + cross_point(1, 1) ^ 2) ^ 0.5

        '旋回終了座標を求める。
        '第１、４象限の時は近いほう
        If target_x >= 0 Then
            If (dist1 <= dist2) Then
                end_of_arc(0) = cross_point(0, 0)
                end_of_arc(1) = cross_point(0, 1)
            Else
                end_of_arc(0) = cross_point(1, 0)
                end_of_arc(1) = cross_point(1, 1)
            End If
        Else
            '第２、３象限の時は遠いほう
            If (dist1 >= dist2) Then
                end_of_arc(0) = cross_point(0, 0)
                end_of_arc(1) = cross_point(0, 1)
            Else
                end_of_arc(0) = cross_point(1, 0)
                end_of_arc(1) = cross_point(1, 1)
            End If
        End If

        '旋回角を求める
        '第１、２象限の時は半時計回り
        If target_y >= 0 Then
            'atan2が使えるように座標変換
            x = -end_of_arc(1) + R(0)
            y = end_of_arc(0)
            turn_q = Atan2(y, x)
        Else
            'atan2が使えるように座標変換
            x = -end_of_arc(1) - R(0)
            y = end_of_arc(0)
            turn_q = -(PI - Atan2(y, x))
        End If

        '直線走行距離を求める
        coo(0, 0) = end_of_arc(0)
        coo(0, 1) = end_of_arc(1)
        coo(1, 0) = target_x
        coo(1, 1) = target_y

        dist2go = calc_2d.dist2points(coo)

    End Sub

    '指定座標へ移動する。
    '座標は現在位置が原点。初めに旋回してから直進する。
    'ゴール地点での方向指定はなし。（まっすぐ向かう）
    'taget_x, taget_y   : 移動目標座標（ｍ単位）
    'turn_r             : 旋回半径（ｍ単位）・・・旋回半径の実測値を与える
    'turn_q             : 旋回部分での旋回量（ラジアン）→ＧＰＳで移動方向を求める時に直進開始位置を求めるのに使用
    'end_of_arc()       : 旋回終了地点の相対座標
    Shared Sub go_coo0(ByVal target_x As Double, ByVal target_y As Double, ByVal turn_r As Double, ByRef turn_q As Double, ByRef end_of_arc() As Double)

        Dim dist0, dist1, dist2, dist2go, center(1, 1), R(1), cross_point(1, 1), coo(1, 1) As Double
        Dim x, y As Double
        Dim dist_run_puls As Integer

        Dim calc_2d As Calc_2D

        calc_2d = New Calc_2D

        '前提条件
        '現在のＥＲ０１の位置をＸ，Ｙ＝０，０の原点とする
        '向きは前方をＸ軸の正とする
        'ＥＲ０１の旋回半径（実測値）
        R(0) = turn_r

        '目標が第3、第4象限の場合→右旋回、旋回中心はX軸の正側
        '目標が第1、第2象限の場合→左旋回、旋回中心はX軸の負側
        '旋回中心の座標
        If target_y <= 0 Then
            center(0, 0) = 0.0 'X
            center(0, 1) = -R(0) 'Y
        Else
            center(0, 0) = 0.0 'X
            center(0, 1) = R(0) 'Y
        End If

        '目標から旋回円への接線を求めるため、２円の交点を求める。
        '一つ目の円は、目標と原点を結ぶ直線上で、その距離から旋回円の半径をマイナスした後
        'の数値の半分
        coo(0, 0) = center(0, 0)
        coo(0, 1) = center(0, 1)
        coo(1, 0) = target_x
        coo(1, 1) = target_y

        dist0 = calc_2d.dist2points(coo)
        R(1) = dist0 / 2.0

        '２つ目の円の中心
        center(1, 0) = (target_x - center(0, 0)) / 2 + center(0, 0)
        center(1, 1) = (target_y - center(0, 1)) / 2 + center(0, 1)

        calc_2d.crcir(center, R, cross_point)

        dist1 = (cross_point(0, 0) ^ 2 + cross_point(0, 1) ^ 2) ^ 0.5
        dist2 = (cross_point(1, 0) ^ 2 + cross_point(1, 1) ^ 2) ^ 0.5

        '旋回終了座標を求める。
        '第１、４象限の時は近いほう
        If target_x >= 0 Then
            If (dist1 <= dist2) Then
                end_of_arc(0) = cross_point(0, 0)
                end_of_arc(1) = cross_point(0, 1)
            Else
                end_of_arc(0) = cross_point(1, 0)
                end_of_arc(1) = cross_point(1, 1)
            End If
        Else
            '第２、３象限の時は遠いほう
            If (dist1 >= dist2) Then
                end_of_arc(0) = cross_point(0, 0)
                end_of_arc(1) = cross_point(0, 1)
            Else
                end_of_arc(0) = cross_point(1, 0)
                end_of_arc(1) = cross_point(1, 1)
            End If
        End If

        '旋回角を求める
        '第１、２象限の時は半時計回り
        If target_y >= 0 Then
            'atan2が使えるように座標変換
            x = -end_of_arc(1) + R(0)
            y = end_of_arc(0)
            turn_q = Atan2(y, x)
        Else
            'atan2が使えるように座標変換
            x = -end_of_arc(1) - R(0)
            y = end_of_arc(0)
            turn_q = -(PI - Atan2(y, x))
        End If

        '直線走行距離を求める
        coo(0, 0) = end_of_arc(0)
        coo(0, 1) = end_of_arc(1)
        coo(1, 0) = target_x
        coo(1, 1) = target_y

        dist2go = calc_2d.dist2points(coo)

        'ＥＲ０１を走らせる
        '旋回
        turn0(turn_q, 3, 1, dist_run_puls, False)
        frmGPSTest.wait_running() '走行完了を待つ

        '障害物を検知した場合の処理
        If frmJoint.flg_running <> 0 Then
            MsgBox("障害物を検出しました。走行を中断します。(go_coo0)")
            Exit Sub
        End If

        '直進
        go_foward(dist2go, 3, 1)
        frmGPSTest.wait_running() '走行完了を待つ

        '障害物を検知した場合の処理
        If frmJoint.flg_running <> 0 Then
            MsgBox("障害物を検出しました。走行を中断します。(go_coo0)")
            Exit Sub
        End If

    End Sub

    '指定距離（ｍ単位）直進する
    Shared Sub go_foward(ByVal dist_run_m As Double, ByVal power_code As Integer, ByVal abort_check As Integer)

        Dim dist_run_puls, power_puls, sttering_puls, steering_newtral As Integer

        dist_run_puls = dist_run_m * 1000 / (PI * 120.0 / 4)
        steering_newtral = Values.joint_data_org(0)

        power_puls = 1870 - power_code * 25
        sttering_puls = steering_newtral

        Values.joint_data_last(0) = sttering_puls
        Values.joint_data_last(1) = power_puls

        SIO.start_moving(abort_check, dist_run_puls, sttering_puls, power_puls)

    End Sub

    '指定距離（ｍ単位）直進する
    Shared Sub back0(ByVal dist_run_m As Double, ByVal power_code As Integer, ByVal abort_check As Integer)

        Dim dist_run_puls, power_puls, sttering_puls As Integer

        '******************************************
        dist_run_puls = 8
        power_puls = frmJoint.accel_newtral + frmJoint.back_plm
        sttering_puls = Values.joint_data_org(0)
        SIO.start_moving(abort_check, dist_run_puls, sttering_puls, power_puls) 'ブレーキ
        System.Threading.Thread.Sleep(100)

        '******************************************
        SIO.stop_moving()
        System.Threading.Thread.Sleep(100)

        '******************************************
        dist_run_puls = dist_run_m * 1000 / (PI * 120.0 / 4)
        power_puls = Values.joint_data_org(1) + frmJoint.back_plm
        sttering_puls = Values.joint_data_org(0)

        Values.joint_data_last(0) = sttering_puls
        Values.joint_data_last(1) = power_puls

        SIO.start_moving(abort_check, dist_run_puls, sttering_puls, power_puls) '後進

    End Sub

    '指定角度（ラジアン）旋回する
    Shared Sub turn0(ByVal direction As Double, ByVal power_code As Integer, ByVal touch_abort As Integer, ByRef dist_run_puls As Integer, ByVal high_resolution As Boolean)

        Dim power_puls, steering_puls, steering_newtral, steering_notch, servo_data As Integer
        Dim base_puls_L, base_puls_R As Double
        Dim head_dir, servo_st_en(6) As Double

        If high_resolution = True Then
            '緩い旋回をする代わりに旋回精度高い
            base_puls_L = Values.turn_count_L2
            base_puls_R = Values.turn_count_R2
            steering_notch = 2
        Else
            '急旋回をする代わりに旋回精度低い
            base_puls_L = Values.turn_count_L1
            base_puls_R = Values.turn_count_R1
            steering_notch = 5
        End If
        steering_newtral = Values.joint_data_org(0)

        '関節パラメータセット（後日必要あれば関節パラメータファイルから読むように変更2009/11/08）
        servo_st_en(0) = -45
        servo_st_en(1) = 630
        servo_st_en(2) = 0
        servo_st_en(3) = 1555
        servo_st_en(4) = 45
        servo_st_en(5) = 2505

        If direction > 0 Then
            '左旋回
            steering_puls = steering_newtral + 125 * steering_notch
            dist_run_puls = Int(Abs(direction) / (2 * PI) * base_puls_L)
            head_dir = 35.0
        Else
            '右旋回
            steering_puls = steering_newtral - 125 * steering_notch
            dist_run_puls = Int(Abs(direction) / (2 * PI) * base_puls_R)
            head_dir = -35.0
        End If
        Values.joint_data_new(0) = steering_puls

        '首を進行方向へ動かす
        servo_data = frmSonicScan.angle2servo(head_dir, servo_st_en)
        Values.joint_data_new(2) = servo_data
        SIO.move_servo(25, Values.joint_data_new, Values.joint_data_last)
        '0.2秒待つ
        System.Threading.Thread.Sleep(200)

        power_puls = 1870 - power_code * 25
        Values.joint_data_last(1) = power_puls

        SIO.start_moving(touch_abort, dist_run_puls, steering_puls, power_puls)

    End Sub

    '障害物回避ルーチン（とりあえずＳｏｎｉｃアボートの対応から書き始める）
    Shared Sub avoid(ByRef dx As Double, ByRef dy As Double, ByRef dq As Double, ByRef ret_flg As Integer)

        '変数定義
        Dim i, j, k, sonic_sum(2), sonic_av(2) As Integer
        Dim scan_width, scan_step, distance(32), scan_devision, dist_run_puls As Integer

        Dim dist_unit As Double
        Dim power_code, dist_limit_puls, dist_limit_raw_psd, touch_abort As Integer

        'Ｓｏｎｉｃスキャン
        'スキャン幅とステップをセットする
        scan_width = 70
        scan_devision = 7
        scan_step = scan_width / scan_devision

        'スキャンを実行する
        Call frmSonicScan.scan_Sonic(scan_width, scan_step, distance)


        'スキャンエリアを３分割して行ける方向を探す
        '右手エリア
        k = scan_devision * 2 / 3
        sonic_sum(0) = 0
        For i = 0 To k - 1
            sonic_sum(0) = sonic_sum(0) + distance(i)
        Next
        sonic_av(0) = sonic_sum(0) / (scan_devision * 2 / 3)
        j = i

        '正面エリア
        k = scan_devision * 2 / 3 * 2
        sonic_sum(1) = 0
        For i = j To k - 1
            sonic_sum(1) = sonic_sum(1) + distance(i)
        Next
        sonic_av(1) = sonic_sum(1) / (scan_devision * 2 / 3)
        j = i

        '左手エリア
        k = scan_devision * 2
        sonic_sum(0) = 0
        For i = 0 To k - 1
            sonic_sum(2) = sonic_sum(2) + distance(i)
        Next
        sonic_av(2) = sonic_sum(2) / (scan_devision * 2 / 3)

        j = 0
        k = -1
        For i = 0 To 2
            If j < sonic_sum(i) Then
                j = sonic_sum(i)
                k = i
            End If
        Next

        '結果の評価（一番すいている方の距離の平均値が２５００ｍｍ以下のときは行き止まりで回避失敗）
        If sonic_av(k) < 2500 Then
            '行き止まりの時はバックしないといけないのでとりあえず回避失敗としておく2009/11/12
            ret_flg = -1 '回避失敗
            Exit Sub
        End If

        '回避旋回実行
        power_code = 4.0
        dist_limit_puls = 800 / 1000 * 11640.1
        SIO.PSD_mm2raw(300, dist_limit_raw_psd)
        touch_abort = 1
        frmJoint.dist_count2 = 0

        If k = 0 Or k = 2 Then
            If k = 0 Then
                dq = -0.79 '右旋回
            Else
                dq = 0.79 '左旋回
            End If

            RoboIO_Move.turn0(dq, power_code, touch_abort, dist_run_puls, False)
        Else
            '旋回中のアボートは正面が抜け道の場合もあるので直進
            If k = 1 Then dq = 0.0 '正面
            dist_unit = 3.0

            RoboIO_Move.go_foward(dist_unit, power_code, touch_abort)
        End If
        frmGPSTest.wait_running() '走行完了を待つ

        ret_flg = 1 '回避動作実施（結果は再評価必用）

    End Sub

    'タッチセンサ用回避ルーチン（バックしてから方向転換）
    Shared Sub avoid2(ByRef dx As Double, ByRef dy As Double, ByRef dq As Double, ByRef ret_flg As Integer)

        '変数定義
        Dim dist_run_puls As Integer
        Dim power_code, touch_abort As Integer

        frmJoint.Timer_moving.Stop()

        'バック(1.5m)
        RoboIO_Move.back0(1.5, 0, 0) '前向きのセンサは全て見ない
        System.Threading.Thread.Sleep(2000)
        frmGPSTest.wait_running() '走行完了を待つ


        frmJoint.Timer_moving.Stop()
        '旋回
        power_code = 4.0
        touch_abort = 1
        frmJoint.dist_count2 = 0

        dq = 45 / 180 * PI '左旋回 45度
        RoboIO_Move.turn0(dq, power_code, touch_abort, dist_run_puls, False) 'タッチセンサのみ有効にして旋回
        frmGPSTest.wait_running() '走行完了を待つ

        ret_flg = 1 '回避動作実施（結果は再評価必用）

    End Sub
End Class
