﻿Imports System.Math
Public Class utils1

    Public Structure SensorTable
        Dim data_number As Integer     'テーブルの要素数
        Dim data_min As Double         '最小値（抵抗値の）
        Dim data_max As Double         '最大値（抵抗値の）
        Dim sensor_table(,) As Double   'テーブル　１列目：抵抗値（ｋΩ）　２列目：温度（℃）
    End Structure

    '測温抵抗体の抵抗値と温度の換算（直線補間）
    'raw_data   ：抵抗値
    Shared Function Hokan(ByVal sensor_table As SensorTable, ByVal raw_data As Integer)

        Dim i As Integer
        Dim a As Double, b As Double, resist_data As Double

        Hokan = 0

        '測定生データ　→　抵抗値変換
        resist_data = 512 / raw_data - 1

        'レンジオーバーの判定
        If resist_data > sensor_table.data_max Then
            Hokan = -999
            Exit Function
        End If
        If resist_data < sensor_table.data_min Then
            Hokan = 999
            Exit Function
        End If

        For i = 0 To sensor_table.data_number - 2

            If resist_data <= sensor_table.sensor_table(i, 0) Then

                'ステップ幅（抵抗値）
                a = sensor_table.sensor_table(i + 1, 0) - sensor_table.sensor_table(i, 0)
                b = resist_data - sensor_table.sensor_table(i, 0)

                Hokan = (sensor_table.sensor_table(i + 1, 1) - sensor_table.sensor_table(i, 1)) * (b / a) + sensor_table.sensor_table(i, 1)

            End If

        Next i

    End Function

    '測温抵抗体の抵抗値と温度の換算テーブル読み込み
    Shared Sub ReadTable(ByRef sensor_table As SensorTable, ByVal input_file As String)

        Dim line_in As String
        Dim work As Object

        '測温抵抗体の温度／抵抗テーブル変数を初期化
        sensor_table.data_number = 0
        sensor_table.data_max = -9999
        sensor_table.data_min = 9999

        FileOpen(3, input_file, OpenMode.Input)


        Do While Not EOF(3)

            line_in = LineInput(3)
            work = line_in.Split(",")

            sensor_table.sensor_table(sensor_table.data_number, 0) = Val(work(0)) '抵抗
            sensor_table.sensor_table(sensor_table.data_number, 1) = Val(work(1)) '温度

            '最大抵抗値　取得
            If sensor_table.data_number = 0 Then
                sensor_table.data_max = sensor_table.sensor_table(sensor_table.data_number, 0)
            End If

            sensor_table.data_number = sensor_table.data_number + 1

        Loop
        FileClose(3)

        '最小抵抗値　取得
        sensor_table.data_min = sensor_table.sensor_table(sensor_table.data_number - 1, 0)

        Exit Sub


    End Sub


    '日付と時刻から重複しないファイル名を発生する
    'filename：ファイル名として使う文字列
    'extention：拡張子など、その目的を表すファイル名の後半部分
    Shared Sub get_unique_fname(ByRef filename As String, ByVal extention As String)

        filename = DateString + "-" + TimeString + extention
        filename = Replace(filename, ":", "-")

    End Sub

    '************************************************************************
    '２回のＧＰＳ測定で距離（ｍ単位）と方向（rad）を計算する
    Shared Sub get_diffrence(ByVal position() As RoboIO_GPS.position, ByRef distance As Double, ByRef direction As Double, ByVal cmd As Integer)

        Dim N, N_diff, E_diff, m_1degN, m_1degE, X, Y As Double
        Dim src_ptr As Integer, dst_ptr As Integer


        'ｃｍｄ＝０：前回位置からの移動距離と方向を求める
        'ｃｍｄ＝１：目標位置までの距離と方向を求める
        If cmd = 0 Then
            src_ptr = 1 '前回測定地点
            dst_ptr = 0 '現在位置
        Else
            src_ptr = 0 '現在位置
            dst_ptr = 2 '目標地点
        End If

        N = position(0).N_deg + position(0).N_min / 60

        'メルカトル図法の地図を広げたイメージで、Ｎは縦軸（Ｙ）Ｅは横軸（Ｘ）とする
        N_diff = (position(dst_ptr).N_deg - position(src_ptr).N_deg)
        N_diff = N_diff + (position(dst_ptr).N_min - position(src_ptr).N_min) / 60

        E_diff = (position(dst_ptr).E_deg - position(src_ptr).E_deg)
        E_diff = E_diff + (position(dst_ptr).E_min - position(src_ptr).E_min) / 60

        '緯度をｍに換算すると
        m_1degN = 111.263283 * 1000.0

        '経度をｍに換算すると
        m_1degE = m_1degN * Cos(N / 180 * PI)

        X = E_diff * m_1degE
        Y = N_diff * m_1degN
        '移動距離計算
        distance = (X ^ 2 + Y ^ 2) ^ 0.5

        '移動方向計算（東をX軸（０度）と見て±３．１４１５（１８０°）で返す
        direction = Atan2(Y, X)

    End Sub

    '既にオープンされているＮＭＥＡセンテンスを含むテキストファイルから、指定の行を取り出す。
    Shared Sub get_NMEA_line(ByVal sentence_keyword As String, ByRef gpstext_work As String, ByRef ret_status As Integer, ByVal file_number As Integer)

        Dim work_word() As String

        ret_status = 1
        Do
            If EOF(file_number) = False Then
                gpstext_work = LineInput(file_number)
            Else
                ret_status = 0
                Exit Sub
            End If
            work_word = gpstext_work.Split(",")
        Loop Until work_word(0) = sentence_keyword

    End Sub

    'ＷＧＳ８４のdd.ddddddd座標をＮＭＥＡのddmm.mmmmに変換する
    Shared Sub WGS2NMEA(ByVal W_Ndeg As String, ByVal W_Edeg As String, ByRef N_Ndeg As String, ByRef N_Edeg As String)
        Dim N_deg, N_min, E_deg, E_min As String
        Dim work_str2() As String

        work_str2 = W_Ndeg.Split(".")
        N_deg = work_str2(0)
        N_min = Format(Val("0." + work_str2(1)) * 60, "0.0000")
        N_min.PadRight(7, "0")

        N_Ndeg = N_deg + N_min

        work_str2 = W_Edeg.Split(".")
        E_deg = work_str2(0)
        E_min = Format(Val("0." + work_str2(1)) * 60, "0.0000")
        E_min.PadRight(7, "0")

        N_Edeg = E_deg + E_min

    End Sub

    'ＮＮＥＡのＧＰＧＧＡセンテンスに含まれるddmm.mmmm座標をオフセットする
    Shared Sub offset_position(ByVal dx As Double, ByVal dy As Double, ByRef position As RoboIO_GPS.position)

        Dim N, E, N_diff, E_diff, m_1degN, m_1degE As Double

        N = position.N_deg + position.N_min / 60
        E = position.E_deg + position.E_min / 60

        '緯度をｍに換算する
        m_1degN = 111.263283 * 1000.0

        '経度をｍに換算する
        m_1degE = m_1degN * Cos(N / 180 * PI)

        'オフセット量を度に換算する
        N_diff = dy / m_1degN
        E_diff = dx / m_1degE

        N = N + N_diff
        E = E + E_diff

        '結果をdd.dddd...からdd mm.mmmm...に変換する
        position.N_deg = Int(N)
        position.N_min = (N - Int(N)) * 60

        position.E_deg = Int(E)
        position.E_min = (E - Int(E)) * 60

    End Sub

End Class
